import { Discovery } from './discovery/Discovery';
import { Logger } from './shared/Logger';
import { SyncedRealm } from "./shared/SyncedRealm";
import { RealmDefinition } from "./realms";
export declare class ServerValidationError extends Error {
}
export declare class ServerConfiguration {
    agreement?: string | {
        name: string;
        email: string;
    };
}
export interface ServerStartParams {
    dataPath: string;
    privateKeyPath?: string;
    publicKeyPath?: string;
    autoKeyGen?: boolean;
    logger?: Logger;
    logLevel?: string;
    address?: string;
    port?: number;
    discovery?: Discovery;
    https?: boolean;
    httpsKeyPath?: string;
    httpsCertChainPath?: string;
    httpsPort?: number;
    httpsAddress?: string;
}
export declare class Server {
    protected runningParams: ServerStartParams;
    config: ServerConfiguration;
    services: any[];
    privateKey: string;
    publicKey: string;
    adminToken: string;
    readonly version: string;
    private _started;
    readonly started: boolean;
    readonly dataPath: string;
    readonly discovery: Discovery;
    readonly logger: Logger;
    readonly publicKeyPath: string;
    readonly privateKeyPath: string;
    private expressApp;
    private httpServer;
    private httpsServer;
    constructor(config?: ServerConfiguration);
    useMiddleware(...args: any[]): Server;
    start(params: ServerStartParams): Promise<void>;
    readonly address: string;
    readonly secureAddress: string;
    shutdown(): Promise<void>;
    addServices(...services: any[]): this;
    addService(service: any | string, config?: object): this;
    removeService(service: any | string): this;
    getService(name: string): any;
    private syncedRealms;
    getSyncedRealm(def: RealmDefinition): SyncedRealm;
    private registerSyncedRealm(remotePath, syncedRealm);
    closeSyncedRealm(remotePath: string): void;
    private upgradeHandler(req, socket, head);
    private injectLogger();
    private addServiceEndpoints();
    private startServices();
    private stopServices();
    private authenticationMiddleware(req, res, next);
    private processException(error);
    private errorHandler(error, req, res, next);
    private validateStartParams(params);
}
